/************************************************************************
* Name:         Module 013: Conert GCA to Transmission Function
* Description:  This module extracts the information of a grating cells
*               array (GCA) from a given LPD and converts it to a transmission
*               function. This transmission function can be used for further
*               investiagations and as additional export option.
*               The user can specify a certain number of phase
*               levels and the desired resolution of the resulting transmission.
* Authors:      Christian Hellmann (LightTrans)
* Version:      2.0 (Dec 10, 2015)
* License:      Some Rights reserved, CC-BY 3.0
*************************************************************************/

using System;
using System.Collections.Generic;
using System.Drawing;

using VirtualLabAPI.UI.BasicUI;
using VirtualLabAPI.Core.LightPath;
using VirtualLabAPI.Core.GeometryDescription;
using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.Modules;
using VirtualLabAPI.Core.OpticalSystems;

public class VLModule : IVLModule {
  /// <summary>
  /// run method
  /// </summary>
  public void Run() {
    //get user defined input
    FormGCAToTransmissionConversion formParameters = new FormGCAToTransmissionConversion();

    if(formParameters.ShowDialog() == System.Windows.Forms.DialogResult.OK) {
      DataArray2D gcaParameters = formParameters.GCAParametersToConvert;
      bool isQuantized = formParameters.IsQuantized;
      int numberHeights = formParameters.NumberHeightLevels;
      VectorD resolution = formParameters.Resolution;

      //perform export
      ComplexAmplitude transmission = convertGCArrayToTransmission(gcaParameters,
                                                                   isQuantized,
                                                                   numberHeights,
                                                                   resolution);
      transmission.ModifyComplexPart(ComplexPart.ExtractPhase);
      Globals.DataDisplay.ShowDocument(transmission, "Grating Cells Array coverted into Transmission function");
     }
  }

  /// <summary>
  /// private help method to perform the conversion from the GCA parameter array to a transmission
  /// </summary>
  /// <param name="gratingParameters">the GCA parameters</param>
  /// <param name="isQuantized">shall the GCA be quantized</param>
  /// <param name="numberHeightLevels">number of height levels, if the GCa shall be quantized</param>
  /// <param name="resolution">the resolution which shall be used for the conversion</param>
  /// <returns>the transmission</returns>
  private ComplexAmplitude convertGCArrayToTransmission(DataArray2D gratingParameters,
                                                        bool isQuantized,
                                                        int numberHeightLevels,
                                                        VectorD resolution) {
    SamplingParameters sParaGCA = new SamplingParameters();
    sParaGCA.SamplingDistance = new VectorD(gratingParameters.SamplingDistance_X, gratingParameters.SamplingDistance_Y);
    sParaGCA.SamplingPoints = new Vector(gratingParameters.NoOfDataPoints_X, gratingParameters.NoOfDataPoints_Y);

    //init array for blazed grating transmission generators
    BlazedGratingTransmission1DGenerator[,] generatorArray = new BlazedGratingTransmission1DGenerator[gratingParameters.NoOfDataPoints_X, gratingParameters.NoOfDataPoints_Y];
    BlazedGratingTransmission1DGenerator defaultGratingGenerator = new BlazedGratingTransmission1DGenerator();
    defaultGratingGenerator.OutsideDefinition = TransmissionOutsideDefinition.InfiniteFunction;
    defaultGratingGenerator.IsQuantized = isQuantized;
    if(isQuantized) {
      defaultGratingGenerator.NumberOfHeightSteps = numberHeightLevels;
    }
    defaultGratingGenerator.ModulatePhase = true;
    defaultGratingGenerator.ModulationDepth_AsDefined = MathFunctions.TwoPi;
    defaultGratingGenerator.UsedIn = SpreadSheetObjectUsedIn.UsedInLightPath;
    defaultGratingGenerator.AdjustModulationDepth(Globals.DefaultWavelength);

    for(int runGeneratorsX = 0; runGeneratorsX < gratingParameters.NoOfDataPoints_X; runGeneratorsX++) {
      for(int runGeneratorsY = 0; runGeneratorsY < gratingParameters.NoOfDataPoints_Y; runGeneratorsY++) {
        BlazedGratingTransmission1DGenerator currentGenerator = (BlazedGratingTransmission1DGenerator)(defaultGratingGenerator.Clone());
        currentGenerator.Period = (gratingParameters.Data[0][runGeneratorsX, runGeneratorsY].Re == double.PositiveInfinity) ? 1e6 : gratingParameters.Data[0][runGeneratorsX, runGeneratorsY].Re;
        currentGenerator.RotationAngle = gratingParameters.Data[2][runGeneratorsX, runGeneratorsY].Re;
        currentGenerator.LateralShift = gratingParameters.Data[1][runGeneratorsX, runGeneratorsY].Re;

        generatorArray[runGeneratorsX, runGeneratorsY] = currentGenerator;
      }
     }

     SamplingParameters sParaTranmission = new SamplingParameters();
     sParaTranmission.SamplingDistance = resolution;
     sParaTranmission.SamplingPointsX = (int)((gratingParameters.NoOfDataPoints_X * gratingParameters.SamplingDistance_X) / resolution.X) + 1;
     sParaTranmission.SamplingPointsY = (int)((gratingParameters.NoOfDataPoints_Y * gratingParameters.SamplingDistance_Y) / resolution.Y) + 1;

     ComplexAmplitude caReturn = new ComplexAmplitude(sParaTranmission, PrecisionMode.Double, true);
     caReturn.ComplexAmplitudeType = ComplexAmplitudeType.JonesMatrixTransmission;
     //fill transmission

     for(int runY = 0; runY < sParaTranmission.SamplingPoints.Y; runY++) {
       for(int runX = 0; runX < sParaTranmission.SamplingPoints.X; runX++) {
         VectorD currPhysicalCoordinateGlobal = CoordinateTransformations.PointFromPixelToPhysicalCoordinates(new Vector(runX, runY),
                                                                                                              sParaTranmission);
         //evaluate the the grating cell for the current position
         VectorD currGratingCellDouble = CoordinateTransformations.GCDPointFromPhysicalToDoublePixelCoordinates(currPhysicalCoordinateGlobal,
                                                                                                                       sParaGCA);
         currGratingCellDouble += new VectorD(.49999999, .4999999);
         Vector currGratingCell = (Vector)currGratingCellDouble;
         currGratingCell.X = Math.Min(currGratingCell.X, sParaGCA.SamplingPointsX - 1);
         currGratingCell.Y = Math.Min(currGratingCell.Y, sParaGCA.SamplingPointsY - 1);
         VectorD physicalPositionRelativeInGratingCell = currPhysicalCoordinateGlobal - CoordinateTransformations.GCDPointFromPixelToPhysicalCoordinates(currGratingCell,
                                                                                                                                                         sParaGCA);

         caReturn.Field[runX, runY] = generatorArray[currGratingCell.X, currGratingCell.Y].GetLocalTransmission(physicalPositionRelativeInGratingCell);
       }
     }
     return caReturn;
  }
}


public class FormGCAToTransmissionConversion : FormDialog {
  #region designer code
  /// <summary>
  /// Required designer variable.
  /// </summary>
  private System.ComponentModel.IContainer components = null;

  /// <summary>
  /// Clean up any resources being used.
  /// </summary>
  /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
  protected override void Dispose(bool disposing) {
    if(disposing && (components != null)) {
      components.Dispose();
    }
    base.Dispose(disposing);
  }

  #region Windows Form Designer generated code
  /// <summary>
  /// Required method for Designer support - do not modify
  /// the contents of this method with the code editor.
  /// </summary>
  private void InitializeComponent() {
    this.buttonSelectLPD = new System.Windows.Forms.Button();
    this.labelResolution = new System.Windows.Forms.Label();
    this.putbResolutionX = new VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox();
    this.buttonOK = new System.Windows.Forms.Button();
    this.buttonCancel = new System.Windows.Forms.Button();
    this.labelLPDSpecification = new System.Windows.Forms.Label();
    this.groupBoxQuantization = new System.Windows.Forms.GroupBox();
    this.nUDNumberHeightLevels = new System.Windows.Forms.NumericUpDown();
    this.labelNumberHeightLevels = new System.Windows.Forms.Label();
    this.checkBoxAppyQuantization = new System.Windows.Forms.CheckBox();
    this.textBoxLogging = new System.Windows.Forms.TextBox();
    this.putbResolutionY = new VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox();
    this.labelX = new System.Windows.Forms.Label();
    this.groupBoxQuantization.SuspendLayout();
    ((System.ComponentModel.ISupportInitialize)(this.nUDNumberHeightLevels)).BeginInit();
    this.SuspendLayout();
    // 
    // buttonSelectLPD
    // 
    this.buttonSelectLPD.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.buttonSelectLPD.Location = new System.Drawing.Point(193, 14);
    this.buttonSelectLPD.Name = "buttonSelectLPD";
    this.buttonSelectLPD.Size = new System.Drawing.Size(75, 23);
    this.buttonSelectLPD.TabIndex = 1;
    this.buttonSelectLPD.Text = "Select";
    this.buttonSelectLPD.UseVisualStyleBackColor = true;
    this.buttonSelectLPD.Click += new System.EventHandler(this.buttonSelectLPD_Click);
    // 
    // labelResolution
    // 
    this.labelResolution.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.labelResolution.AutoSize = true;
    this.labelResolution.Location = new System.Drawing.Point(21, 184);
    this.labelResolution.Name = "labelResolution";
    this.labelResolution.Size = new System.Drawing.Size(57, 13);
    this.labelResolution.TabIndex = 2;
    this.labelResolution.Text = "Resolution";
    // 
    // putbResolutionX
    // 
    this.putbResolutionX.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.putbResolutionX.CausesValidation = false;
    this.putbResolutionX.Location = new System.Drawing.Point(108, 181);
    this.putbResolutionX.MaximumSize = new System.Drawing.Size(300, 24);
    this.putbResolutionX.MinimumDouble = 0D;
    this.putbResolutionX.MinimumInteger = 0;
    this.putbResolutionX.MinimumSize = new System.Drawing.Size(0, 24);
    this.putbResolutionX.MinimumValueIsAllowed = false;
    this.putbResolutionX.Name = "putbResolutionX";
    this.putbResolutionX.Size = new System.Drawing.Size(107, 24);
    this.putbResolutionX.TabIndex = 3;
    this.putbResolutionX._DoubleClickHandler += new VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox.BoxMouseDoubleClickHandler(this.putbResolutionX__DoubleClickHandler);
    // 
    // buttonOK
    // 
    this.buttonOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.buttonOK.DialogResult = System.Windows.Forms.DialogResult.OK;
    this.buttonOK.Location = new System.Drawing.Point(193, 214);
    this.buttonOK.Name = "buttonOK";
    this.buttonOK.Size = new System.Drawing.Size(75, 23);
    this.buttonOK.TabIndex = 5;
    this.buttonOK.Text = "&Ok";
    this.buttonOK.UseVisualStyleBackColor = true;
    // 
    // buttonCancel
    // 
    this.buttonCancel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
    this.buttonCancel.Location = new System.Drawing.Point(274, 214);
    this.buttonCancel.Name = "buttonCancel";
    this.buttonCancel.Size = new System.Drawing.Size(75, 23);
    this.buttonCancel.TabIndex = 6;
    this.buttonCancel.Text = "Cancel";
    this.buttonCancel.UseVisualStyleBackColor = true;
    // 
    // labelLPDSpecification
    // 
    this.labelLPDSpecification.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.labelLPDSpecification.AutoSize = true;
    this.labelLPDSpecification.Location = new System.Drawing.Point(21, 19);
    this.labelLPDSpecification.Name = "labelLPDSpecification";
    this.labelLPDSpecification.Size = new System.Drawing.Size(128, 13);
    this.labelLPDSpecification.TabIndex = 7;
    this.labelLPDSpecification.Text = "Lightpath containing GCA";
    // 
    // groupBoxQuantization
    // 
    this.groupBoxQuantization.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.groupBoxQuantization.Controls.Add(this.nUDNumberHeightLevels);
    this.groupBoxQuantization.Controls.Add(this.labelNumberHeightLevels);
    this.groupBoxQuantization.Controls.Add(this.checkBoxAppyQuantization);
    this.groupBoxQuantization.Location = new System.Drawing.Point(8, 75);
    this.groupBoxQuantization.Name = "groupBoxQuantization";
    this.groupBoxQuantization.Size = new System.Drawing.Size(343, 100);
    this.groupBoxQuantization.TabIndex = 8;
    this.groupBoxQuantization.TabStop = false;
    this.groupBoxQuantization.Text = "Quantization";
    // 
    // nUDNumberHeightLevels
    // 
    this.nUDNumberHeightLevels.Location = new System.Drawing.Point(253, 59);
    this.nUDNumberHeightLevels.Minimum = new decimal(new int[] {
            2,
            0,
            0,
            0});
    this.nUDNumberHeightLevels.Name = "nUDNumberHeightLevels";
    this.nUDNumberHeightLevels.Size = new System.Drawing.Size(84, 20);
    this.nUDNumberHeightLevels.TabIndex = 5;
    this.nUDNumberHeightLevels.TextAlign = System.Windows.Forms.HorizontalAlignment.Right;
    this.nUDNumberHeightLevels.Value = new decimal(new int[] {
            2,
            0,
            0,
            0});
            // 
            // labelNumberHeightLevels
            // 
    this.labelNumberHeightLevels.AutoSize = true;
    this.labelNumberHeightLevels.Location = new System.Drawing.Point(13, 61);
    this.labelNumberHeightLevels.Name = "labelNumberHeightLevels";
    this.labelNumberHeightLevels.Size = new System.Drawing.Size(105, 13);
    this.labelNumberHeightLevels.TabIndex = 3;
    this.labelNumberHeightLevels.Text = "Number Height Levels";
    // 
    // checkBoxAppyQuantization
    // 
    this.checkBoxAppyQuantization.AutoSize = true;
    this.checkBoxAppyQuantization.Location = new System.Drawing.Point(16, 30);
    this.checkBoxAppyQuantization.Name = "checkBoxAppyQuantization";
    this.checkBoxAppyQuantization.Size = new System.Drawing.Size(114, 17);
    this.checkBoxAppyQuantization.TabIndex = 1;
    this.checkBoxAppyQuantization.Text = "Apply Quantization";
    this.checkBoxAppyQuantization.UseVisualStyleBackColor = true;
    this.checkBoxAppyQuantization.CheckedChanged += new System.EventHandler(this.checkBoxAppyQuantization_CheckedChanged);
    // 
    // textBoxLogging
    // 
    this.textBoxLogging.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.textBoxLogging.Location = new System.Drawing.Point(24, 49);
    this.textBoxLogging.Name = "textBoxLogging";
    this.textBoxLogging.ReadOnly = true;
    this.textBoxLogging.Size = new System.Drawing.Size(325, 20);
    this.textBoxLogging.TabIndex = 9;
    // 
    // putbResolutionY
    // 
    this.putbResolutionY.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.putbResolutionY.CausesValidation = false;
    this.putbResolutionY.Location = new System.Drawing.Point(243, 181);
    this.putbResolutionY.MaximumSize = new System.Drawing.Size(300, 24);
    this.putbResolutionY.MinimumDouble = 0D;
    this.putbResolutionY.MinimumInteger = 0;
    this.putbResolutionY.MinimumSize = new System.Drawing.Size(0, 24);
    this.putbResolutionY.MinimumValueIsAllowed = false;
    this.putbResolutionY.Name = "putbResolutionY";
    this.putbResolutionY.Size = new System.Drawing.Size(107, 24);
    this.putbResolutionY.TabIndex = 10;
    this.putbResolutionY._DoubleClickHandler += new VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox.BoxMouseDoubleClickHandler(this.putbResolutionY__DoubleClickHandler);
    // 
    // labelX
    // 
    this.labelX.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
    this.labelX.AutoSize = true;
    this.labelX.Location = new System.Drawing.Point(223, 184);
    this.labelX.Name = "labelX";
    this.labelX.Size = new System.Drawing.Size(12, 13);
    this.labelX.TabIndex = 11;
    this.labelX.Text = "x";
    // 
    // FormGCAToTransmissionConversion
    // 
    this.AcceptButton = this.buttonOK;
    this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
    this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
    this.ClientSize = new System.Drawing.Size(361, 249);
    this.Controls.Add(this.labelX);
    this.Controls.Add(this.putbResolutionY);
    this.Controls.Add(this.textBoxLogging);
    this.Controls.Add(this.groupBoxQuantization);
    this.Controls.Add(this.labelLPDSpecification);
    this.Controls.Add(this.buttonCancel);
    this.Controls.Add(this.buttonOK);
    this.Controls.Add(this.putbResolutionX);
    this.Controls.Add(this.labelResolution);
    this.Controls.Add(this.buttonSelectLPD);
    this.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F);
    this.Name = "FormGCAToTransmissionConversion";
    this.Text = "GCA to Transmission Parameters";
    this.Load += new System.EventHandler(this.FormGCAToTransmissionConversion_Load);
    this.groupBoxQuantization.ResumeLayout(false);
    this.groupBoxQuantization.PerformLayout();
    ((System.ComponentModel.ISupportInitialize)(this.nUDNumberHeightLevels)).EndInit();
    this.ResumeLayout(false);
    this.PerformLayout();
  }

  #endregion

  private System.Windows.Forms.Button buttonSelectLPD;
  private System.Windows.Forms.Label labelResolution;
  private VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox putbResolutionX;
  private System.Windows.Forms.Button buttonOK;
  private System.Windows.Forms.Button buttonCancel;
  private System.Windows.Forms.Label labelLPDSpecification;
  private System.Windows.Forms.GroupBox groupBoxQuantization;
  private System.Windows.Forms.NumericUpDown nUDNumberHeightLevels;
  private System.Windows.Forms.Label labelNumberHeightLevels;
  private System.Windows.Forms.CheckBox checkBoxAppyQuantization;
  private System.Windows.Forms.TextBox textBoxLogging;
  private VirtualLabAPI.UI.BasicUI.PhysicalUnitTextBox putbResolutionY;
  private System.Windows.Forms.Label labelX;
  #endregion
  
  #region constructors, stored data and properties
  public FormGCAToTransmissionConversion() {
    InitializeComponent();

    this.addControlAffectedByLocking(this.textBoxLogging);
    //init GUI settings
    this.putbResolutionX.DoubleValue = this.putbResolutionY.DoubleValue = 10e-6;
    //trigger GUI updates
    checkBoxAppyQuantization_CheckedChanged(this, EventArgs.Empty);
    performLogging("No Grating Cell Array selected.", true);
  }

  /// <summary>
  /// internal stored GCA parameters
  /// </summary>
  private DataArray2D _gcaParameters = null;

  /// <summary>
  /// public property to get GCA parameters
  /// </summary>
  public DataArray2D GCAParametersToConvert {
    get {
      return _gcaParameters;
    }
  }

  /// <summary>
  /// public property to get whether the transmission shall be quantized
  /// </summary>
  public bool IsQuantized {
    get {
      return this.checkBoxAppyQuantization.Checked;
    }
  }

  /// <summary>
  /// public property to get the number of height levels
  /// </summary>
  public int NumberHeightLevels {
    get {
      return (int)(this.nUDNumberHeightLevels.Value);
    }
  }

  /// <summary>
  /// public property to get the resolution which shall be used for conversion
  /// </summary>
  public VectorD Resolution {
    get {
      return new VectorD(this.putbResolutionX.DoubleValue,
                         this.putbResolutionY.DoubleValue);
    }
  }
  #endregion

  #region methods
  /// <summary>
  /// suport message that perform error/message logging + locking of dialog
  /// </summary>
  /// <param name="Message">the message that shall be logged</param>
  /// <param name="isError">if error, the dialog is locked</param>
  private void performLogging(string Message,
                              bool isError) {
    this.textBoxLogging.Text = Message;
    if(isError) {
      this.textBoxLogging.ForeColor = Color.Red;
    }
    else {
      this.textBoxLogging.ForeColor = Color.Black;
    }

    this.ControlLockingStateHasChanged(this.textBoxLogging, isError);
  }
  #endregion

  #region eventhandling
  /// <summary>
  /// eventhandling for changing the state for quantization
  /// </summary>
  /// <param name="sender">the sender of the event</param>
  /// <param name="e">the arguments of the event</param>
  private void checkBoxAppyQuantization_CheckedChanged(object sender, EventArgs e) {
    this.labelNumberHeightLevels.Enabled = checkBoxAppyQuantization.Checked;
    this.nUDNumberHeightLevels.Enabled = checkBoxAppyQuantization.Checked;
  }

  /// <summary>
  /// eventhandling for double clicking into the physical unit text box for X resolution
  /// </summary>
  /// <param name="sender">the sender of the event</param>
  private void putbResolutionX__DoubleClickHandler(object sender) {
    this.putbResolutionX.DoubleValue = this.putbResolutionY.DoubleValue;
  }

  /// <summary>
  /// eventhandling for double clicking into the physical unit text box for Y resolution
  /// </summary>
  /// <param name="sender">the sender of the event</param>
  private void putbResolutionY__DoubleClickHandler(object sender) {
    this.putbResolutionY.DoubleValue = this.putbResolutionX.DoubleValue;
  }

  /// <summary>
  /// eventhandling for clicking on the select LPD button
  /// </summary>
  /// <param name="sender">the sender of the event</param>
  /// <param name="e">the arguments of the event</param>
  private void buttonSelectLPD_Click(object sender, EventArgs e) {
    int index = -1;
    Lightpath lpdGCA = (Lightpath)Globals.ActiveDocumentHistory.BrowseLastDocuments(DocumentFilter.LightPathDiagram,
                                                                                    out index,
                                                                                    "Select LPD to extract GCA information");

    if(lpdGCA == null) {
      performLogging("No LPD selected.", true);
      //error handling
      return;
    }

    if(lpdGCA.DedicatedToolbox != VirtualLabAPI.Core.VLCopyProtection.ToolBoxes.Lighting_Toolbox) {
      performLogging("No LPD containing a GCA element selected.", true);
      //error handling
      return;
    }

    try{
      BoundaryOperator boGCA1 = ((PlateLPE)lpdGCA[1]).GeometryOfComponent.ListOfBoundarySegments[0].Operator;
      BoundaryOperator boGCA2 = ((PlateLPE)lpdGCA[1]).GeometryOfComponent.ListOfBoundarySegments[1].Operator;

      if((boGCA1 is BoundaryOperatorGratingCellsArray) &&
         (boGCA1.UseDuringPropagation) &&
         (boGCA2 is BoundaryOperatorGratingCellsArray) &&
         (boGCA2.UseDuringPropagation)) {
           performLogging("Selected LPD contains more than one GCA element.", true);
           return;
      }

      if((boGCA1 is BoundaryOperatorGratingCellsArray) &&
         (boGCA1.UseDuringPropagation)) {
           this._gcaParameters = (DataArray2D)(((BoundaryOperatorGratingCellsArray)boGCA1).CellsData.Clone());
           performLogging("Parameters of Grating Cells Array imported successfully.", false);
           return;
      }

      if((boGCA2 is BoundaryOperatorGratingCellsArray) &&
         (boGCA2.UseDuringPropagation)) {
           this._gcaParameters = (DataArray2D)(((BoundaryOperatorGratingCellsArray)boGCA2).CellsData.Clone());
           performLogging("Parameters of Grating Cells Array imported successfully.", false);
           return;
      }
      performLogging("No Grating Cells Array to imported found.", true);

    } catch {
      performLogging("No valid Grating Cells Array to imported found.", true);
    }
  }
  
  
  /// <summary>
  /// eventhandling for loading the form
  /// </summary>
  /// <param name="sender">the sender of the event</param>
  /// <param name="e">the arguments of the event</param>
  private void FormGCAToTransmissionConversion_Load(object sender, EventArgs e) {
    performLogging("No Grating Cell Array selected.", true);
  }
  #endregion        
}